/*
 * 爱组搭，低代码组件化开发平台
 * ------------------------------------------
 * 受知识产权保护，请勿删除版权申明，开发平台不允许做非法网站，后果自负
 */
package com.aizuda.boot.modules.system.service.impl;

import com.aizuda.boot.modules.system.entity.SysUser;
import com.aizuda.boot.modules.system.entity.param.AssignDepartmentsParam;
import com.aizuda.boot.modules.system.entity.param.AssignRolesParam;
import com.aizuda.boot.modules.system.entity.param.ResetPasswordParam;
import com.aizuda.boot.modules.system.entity.param.SysUserParam;
import com.aizuda.boot.modules.system.entity.vo.SysUserRelIdsVO;
import com.aizuda.boot.modules.system.entity.vo.SysUserVO;
import com.aizuda.boot.modules.system.mapper.SysUserMapper;
import com.aizuda.boot.modules.system.service.ISysUserDepartmentService;
import com.aizuda.boot.modules.system.service.ISysUserRoleService;
import com.aizuda.boot.modules.system.service.ISysUserService;
import com.aizuda.common.toolkit.RegexUtils;
import com.aizuda.core.api.ApiAssert;
import com.aizuda.service.service.BaseServiceImpl;
import com.baomidou.kisso.common.encrypt.MD5;
import com.baomidou.kisso.common.encrypt.MD5Salt;
import com.baomidou.kisso.common.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import jakarta.annotation.Resource;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;

/**
 * 系统用户 服务实现类
 *
 * @author 青苗
 * @since 2021-11-03
 */
@Service
public class SysUserServiceImpl extends BaseServiceImpl<SysUserMapper, SysUser> implements ISysUserService {
    @Resource
    private ISysUserRoleService sysUserRoleService;
    @Resource
    private ISysUserDepartmentService sysUserDepartmentService;

    @Override
    public Page<SysUser> page(Page<SysUser> page, SysUserVO vo) {
        Page<SysUser> sysUserPage = baseMapper.selectPageByVO(page, vo);
        if (sysUserPage.getTotal() > 0) {
            // 对外隐藏属性
            sysUserPage.getRecords().forEach(t -> {
                t.setPassword(null);
                t.setSalt(null);
            });
        }
        return sysUserPage;
    }

    @Override
    public List<SysUser> list20ByUsername(String username) {
        LambdaQueryWrapper<SysUser> lqw = Wrappers.lambdaQuery();
        if (StringUtils.isNotBlank(username)) {
            lqw.like(SysUser::getUsername, username);
            lqw.or().like(SysUser::getRealName, username);
        }
        return super.page(Page.of(1, 20, false), lqw).getRecords();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean save(SysUserParam param) {
        ApiAssert.fail(!RegexUtils.matches("(^[A-Za-z]{6,50}$)|(^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{6,50}$)", param.getUsername()),
                "用户名，必须是 6 到 50 位 字母 或者 字母数字混合 字符串");
        this.checkPassword(param.getPassword());
        // 检查登录账号是否存在
        this.checkExists(Wrappers.<SysUser>lambdaQuery().select(SysUser::getId).eq(SysUser::getUsername, param.getUsername()), "登录账号已存在");
        param.setSalt(RandomUtil.getCharacterAndNumber(8));
        param.setPassword(this.encodePassword(param.getUsername(), param.getSalt(), param.getPassword()));
        ApiAssert.fail(!super.save(param), "用户信息保存失败");
        if (CollectionUtils.isNotEmpty(param.getRoleIds())) {
            ApiAssert.fail(!this.assignRoles(param), "角色分配保存失败");
        }
        if (CollectionUtils.isNotEmpty(param.getDepartmentIds())) {
            ApiAssert.fail(!this.assignDepartments(param), "部门分配保存失败");
        }
        return true;
    }

    protected void checkPassword(String password) {
        ApiAssert.fail(!RegexUtils.isPassword(password), "登录密码必须为6-20位大小写字母数字特殊字符组合");
    }

    protected String encodePassword(String username, String salt, String password) {
        return MD5Salt.encode(username + salt, MD5.toMD5(password));
    }

    protected boolean assignRoles(SysUserParam param) {
        // 分配角色
        AssignRolesParam assignRolesParam = new AssignRolesParam();
        assignRolesParam.setRoleIds(param.getRoleIds());
        assignRolesParam.setUserIds(Arrays.asList(param.getId()));
        return sysUserRoleService.assignRoles(assignRolesParam);
    }

    protected boolean assignDepartments(SysUserParam param) {
        // 分配部门
        AssignDepartmentsParam assignDepartmentsParam = new AssignDepartmentsParam();
        assignDepartmentsParam.setDepartmentIds(param.getDepartmentIds());
        assignDepartmentsParam.setUserIds(Arrays.asList(param.getId()));
        return sysUserDepartmentService.assignDepartments(assignDepartmentsParam);
    }

    @Override
    public boolean updateById(SysUserParam param) {
        ApiAssert.fail(!super.updateById(param), "用户信息保存失败");
        // 登录账号，密码，盐不允许更新
        param.setUsername(null);
        param.setPassword(null);
        param.setSalt(null);
        ApiAssert.fail(!this.assignRoles(param), "角色分配保存失败");
        ApiAssert.fail(!this.assignDepartments(param), "部门分配保存失败");
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean resetPassword(ResetPasswordParam param) {
        this.checkPassword(param.getPassword());
        List<Long> ids = param.getIds();
        for (Long id : ids) {
            SysUser sysUser = super.getOne(Wrappers.<SysUser>lambdaQuery()
                    .select(SysUser::getUsername, SysUser::getSalt)
                    .eq(SysUser::getId, id));
            if (null == sysUser) {
                continue;
            }
            SysUser temp = new SysUser();
            temp.setId(id);
            temp.setPassword(this.encodePassword(sysUser.getUsername(), sysUser.getSalt(), param.getPassword()));
            super.updateById(temp);
        }
        return true;
    }

    @Override
    public SysUserRelIdsVO getRelIdsById(Long id) {
        SysUserRelIdsVO vo = new SysUserRelIdsVO();
        vo.setRoleIds(sysUserRoleService.listRoleIdsByUserId(id));
        vo.setDepartmentIds(sysUserDepartmentService.listDepartmentIdsByUserId(id));
        return vo;
    }
}
